"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
require("isomorphic-fetch");
function createApolloFetch(params) {
    var _this = this;
    if (params === void 0) { params = {}; }
    var uri = params.uri, customFetch = params.customFetch;
    var _uri = uri || '/graphql';
    var _middlewares = [];
    var _afterwares = [];
    var applyMiddlewares = function (requestAndOptions) {
        return new Promise(function (resolve, reject) {
            var request = requestAndOptions.request, options = requestAndOptions.options;
            var queue = function (funcs, scope) {
                var next = function () {
                    if (funcs.length > 0) {
                        var f = funcs.shift();
                        if (f) {
                            f.applyMiddleware.apply(scope, [{ request: request, options: options }, next]);
                        }
                    }
                    else {
                        resolve({
                            request: request,
                            options: options,
                        });
                    }
                };
                next();
            };
            queue(_middlewares.slice(), _this);
        });
    };
    var applyAfterwares = function (_a) {
        var response = _a.response, options = _a.options;
        return new Promise(function (resolve, reject) {
            var responseObject = { response: response, options: options };
            var queue = function (funcs, scope) {
                var next = function () {
                    if (funcs.length > 0) {
                        var f = funcs.shift();
                        if (f) {
                            f.applyAfterware.apply(scope, [responseObject, next]);
                        }
                    }
                    else {
                        resolve(responseObject);
                    }
                };
                next();
            };
            queue(_afterwares.slice(), _this);
        });
    };
    var callFetch = function (_a) {
        var request = _a.request, options = _a.options;
        var opts = __assign({ body: JSON.stringify(request), method: 'POST' }, options, { headers: __assign({ Accept: '*/*', 'Content-Type': 'application/json' }, options.headers) });
        return customFetch ? customFetch(_uri, opts) : fetch(_uri, opts);
    };
    var apolloFetch = Object.assign(function (request) {
        var options = {};
        return applyMiddlewares({
            request: request,
            options: options,
        })
            .then(callFetch)
            .then(function (response) { return response.text()
            .then(function (raw) {
            try {
                var parsed = JSON.parse(raw);
                return __assign({}, response, { raw: raw, parsed: parsed });
            }
            catch (e) {
                return __assign({}, response, { raw: raw, parsed: null });
            }
        })
            .catch(function (error) {
            var httpError = new Error("Network request failed with status " + response.status + " - \"" + response.statusText + "\"");
            httpError.response = response;
            httpError.parseError = error;
            throw httpError;
        }); })
            .then(function (response) { return applyAfterwares({
            response: response,
            options: options,
        }); })
            .then(function (_a) {
            var response = _a.response;
            if (response.parsed) {
                return __assign({}, response.parsed);
            }
            else {
                var jsonError = new Error("Network request failed to return valid JSON");
                jsonError.response = response;
                jsonError.raw = response.raw;
                throw jsonError;
            }
        });
    }, {
        use: function (middlewares) {
            middlewares.map(function (middleware) {
                if (typeof middleware.applyMiddleware === 'function') {
                    _middlewares.push(middleware);
                }
                else {
                    throw new Error('Middleware must implement the applyMiddleware function');
                }
            });
            return apolloFetch;
        },
        useAfter: function (afterwares) {
            afterwares.map(function (afterware) {
                if (typeof afterware.applyAfterware === 'function') {
                    _afterwares.push(afterware);
                }
                else {
                    throw new Error('Afterware must implement the applyAfterware function');
                }
            });
            return apolloFetch;
        },
        _middlewares: _middlewares,
        _afterwares: _afterwares,
    });
    return apolloFetch;
}
exports.createApolloFetch = createApolloFetch;
//# sourceMappingURL=apollo-fetch.js.map